/*
Copyright (c) 2006, Dennis M. Sosnoski
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.
 * Neither the name of JiBX nor the names of its contributors may be used
   to endorse or promote products derived from this software without specific
   prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package org.jibx.typeinfo;

import java.util.ArrayList;

/**
 * Holder for package information.
 *
 * @author Dennis M. Sosnoski
 */
public class PackageDescription
{
    /** Containing package information. */
    private final PackageDescription m_containingPackage;
    
    /** External form package prefix (including trailing '.', if needed). */
    private final String m_externalPrefix;
    
    /** Internal form package prefix (including trailing '/', if needed). */
    private final String m_internalPrefix;
    
    /** Classes from this package. */
    private final ArrayList m_classes;
    
    /** Child packages of this package. */
    private final ArrayList m_childPackages;
    
    /**
     * Constructor.
     * 
     * @param name package simple name (without containing packages)
     * @param pack containing package (<code>null</code> if this is the default
     * package)
     */
    public PackageDescription(String name, PackageDescription pack) {
        m_containingPackage = pack;
        if (pack == null) {
            m_externalPrefix = m_internalPrefix = "";
        } else {
            m_externalPrefix = pack.m_externalPrefix + name + '.';
            m_internalPrefix = pack.m_internalPrefix + name + '/';
        }
        m_classes = new ArrayList();
        m_childPackages = new ArrayList();
    }
    
    //
    // Access methods
    
    /**
     * Get containing package.
     *
     * @return containing package (<code>null</code> if this is the default
     * package)
     */
    public PackageDescription getContainingPackage() {
        return m_containingPackage;
    }
    
    /**
     * Get package name in internal form.
     *
     * @return internal name
     */
    public String getInternalPrefix() {
        return m_internalPrefix;
    }
    
    /**
     * Get package name in external form.
     *
     * @return name
     */
    public String getExternalPrefix() {
        return m_externalPrefix;
    }
    
    /**
     * Get (modifiable) class list.
     * 
     * @return class list
     */
    public ArrayList getClasses() {
        return m_classes;
    }
    
    /**
     * Get (modifiable) child package list.
     * 
     * @return child list
     */
    public ArrayList getChildPackages() {
        return m_childPackages;
    }
}